import pyglet as p
from pyglet.window import key
from pyglet.gl import *
import sys

win = p.window.Window(800, 600, vsync=False)
#I have to create a context, therefore a window, before I can load any resources.
#This would be done automatically by Pyglet, but it was disbled.

from . import levels
from . import objects

#p.options['shadow_window'] = False
#This had to go on the run_game module, apparently????
#p.options['debug'] = False

###http://www.niksula.hut.fi/~hkankaan/Homepages/gravity.html

######CONSTANTS
arrow = p.resource.image('sword2.png')
i_arrow = arrow.get_transform(flip_x = True)
i_arrow.anchor_x = 100
glEnable(GL_BLEND)
glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA) #Why am I still blitting?
glClearColor(4/255, 20/255, 35/255, 1)

tile_size = 50 #I don't want this here
left_padding = 100 #Nor this
bottom_padding = 50 #Or this!

#Things will be drawn padded padding to the left and there will be nothing
#on the right to draw

class Game():
    def __init__(self):
        try:
            NoSaveFile = FileNotFoundError
        except:
            NoSaveFile = IOError #Support for Py3.2 and less.

        try:
            with open('save') as f:
                self.curr_level = int(f.read())
        except NoSaveFile:
            with open('save', 'w') as f:
                f.write('1')
            self.curr_level = 1
        self.curr_line = 1 #This is the line where the swords are.
                           #Shouldn't it be... 0???? D:
        try:
            self.curr_level = int(sys.argv[1])
            #Debugging / cheaty way to get to any level.
        except: ...

        self.has_died = False
        self.game_over = False

        win.set_handlers(base_handler)
        win.push_handlers(game_handler)
        #SO THAT IS HOW THIS WORKS!!! The window has a _STACK_ of handlers.
        #You push to the TOP and you pop the TOP.

    def next_level(self, *a):
        self.curr_level += 1
        with open('save', 'w') as f:
            f.write(str(self.curr_level))
        self.reset()

    def reset(self):
        objects.reset()
        self.label = None
        if self.has_died:
            self.has_died = not self.has_died
            win.push_handlers(game_handler)
        lv = 'level' + str(self.curr_level)
        try:
            self.level = objects.Level(getattr(levels, lv))
        except AttributeError: #Out of levels
            self.game_over = True
            win.pop_handlers()
            with open('save', 'w') as f:
                f.write('1')
            message = 'Game over!\nThank you for playing!\nProgress will be reset.'
            self.label = p.text.Label(message, x=win.width//2, y=win.height//2,
            anchor_x='center', anchor_y='center', font_size=40,
            color=(255, 255, 255, 255), multiline=True, width=550)

    def end_game(self):
        self.has_died = True
        win.pop_handlers()
        self.label = p.text.Label('Press R to restart.', x=win.width//2, y=win.height//2,
        anchor_x='center', font_size=40, color=(255, 255, 255, 255))

    def update(self, *a):
        '''The only change that occurs in the game is when things fall.
        When something falls it sets level.keylock to True. To keep things going
        update is rescheduled. If we schedule update before checking if the game
        has ended, however, we'll run into trouble.'''
        state = self.level.update()
        if state == 'end':
            self.end_game()
        elif state == 'win':
            #play sound? display message?
            p.clock.schedule_once(self.next_level, 1)
        elif self.level.keylock:
            p.clock.schedule_once(self.update, 0.25)


class BasicHandler():
    def on_key_press(self, symbol, modifiers):
        if symbol == key.R:
            game.reset()

class GameHandler():
    def on_key_press(self, symbol, modifiers):
        if not game.level.keylock:
            if symbol == key.LEFT or symbol == key.A:
                game.level[game.curr_line].rotate(-1)
            elif symbol == key.RIGHT or symbol == key.D:
                game.level[game.curr_line].rotate(1)
            game.update()
            #Things only change when we move left or right.

        if symbol == key.UP or symbol == key.W:
            if game.curr_line < game.level.mapsize-1: #How big will the map be?
                game.curr_line += 1
        elif symbol == key.DOWN or symbol == key.S:
            if game.curr_line != 0:
                game.curr_line -= 1
    
            # else: ...
            #     #Play EEENK sound

@win.event
def on_draw():
    win.clear()
    objects.batch.draw()
    try:
        game.label.draw()
    except AttributeError: ... #No label or it is None

    if not game.game_over:
        arrow.blit(0, bottom_padding + game.curr_line*tile_size)
        i_arrow.blit(win.width, bottom_padding + game.curr_line*tile_size)
        #Why am I blitting? I don't know!



base_handler = BasicHandler()
game_handler = GameHandler()
#This could go inside Game.
game = Game()

def run():
    game.reset()
    p.app.run()